/*
 * spi.c
 *
 *  Created on: Jan 23, 2015
 *      Author: Lijo Francis
 */

#include "spi.h"
#include "MK40DZ10.h"
void clear_flags(void);


uint8_t spi_transfer_one_byte(uint8_t value, bool end)
{
    if(end)
        SPI0_PUSHR =// SPI_PUSHR_CONT_MASK |
                     SPI_PUSHR_EOQ_MASK  |
                     SPI_PUSHR_PCS(1<<0) |
                     (value);
    else
        SPI0_PUSHR = SPI_PUSHR_CONT_MASK |
                     SPI_PUSHR_PCS(1<<0) |
                     (value);

    while((SPI0_SR & SPI_SR_TCF_MASK)==0);
    SPI0_SR |= SPI_SR_TCF_MASK;
    return SPI0_POPR&0xff;
}
void register_write(unsigned char* data,unsigned char bytesnum)
{
	unsigned char byte = 0;
    for(byte = 0; byte < bytesnum-1; byte++)
	spi_transfer_one_byte(data[byte],0);
	spi_transfer_one_byte(data[byte],1);

}
void register_read(unsigned char* data,unsigned char bytesnum)
{

	unsigned char byte = 0;
    for(byte = 0; byte < bytesnum-1; byte++)
    data[byte]=spi_transfer_one_byte(data[byte],0);
    data[byte]=spi_transfer_one_byte(data[byte],1);
}
void SPI0_mstrInit()
{

	SIM_SCGC6 |= SIM_SCGC6_DSPI0_MASK;	//SPI0 Clock Gate enabled

	SIM_SCGC5 |= SIM_SCGC5_PORTC_MASK;	//Port A Clock enabled

	PORTC_PCR4 = PORT_PCR_MUX(0x02); 	//SPI0_SIN //B59 in tower module

	PORTC_PCR5 =  PORT_PCR_MUX(0x02); 	//SPI0_SOUT //B60 in tower module

	PORTC_PCR6 =  PORT_PCR_MUX(0x02); 	//SPI0_SCK //A35 in tower module

	PORTC_PCR7 = PORT_PCR_MUX(0x02); 	//SPI0_PCS0  //B35 in tower module

	SPI0_MCR = SPI_MCR_MSTR_MASK |  	//DSPI is in Master mode.
				SPI_MCR_ROOE_MASK |  	//Incoming data is shifted into the shift register.
				SPI_MCR_PCSIS(0x01) |	//The inactive state of PCSx is high.
				SPI_MCR_CLR_TXF_MASK |	//Clear the TX FIFO counter.
				SPI_MCR_CLR_RXF_MASK |  //Clear the RX FIFO counter.
				SPI_MCR_HALT_MASK;		//Stop transfers

	SPI0_MCR |= SPI_MCR_DIS_TXF_MASK |	//Tx FIFO is disabled.
				SPI_MCR_DIS_RXF_MASK;    //Rx FIFO is disabled.	;


	SPI0_SR |= SPI_SR_TCF_MASK |	//Clear transfer complete flag
			   SPI_SR_EOQF_MASK |	//Clear End of Queue Flag
			   SPI_SR_TFUF_MASK |	//CLear Transmit FIFO Underflow Flag
			   SPI_SR_TFFF_MASK ;	//Transmit FIFO Fill Flag


	//SPI0_CTAR0 = SPI_CTAR_FMSZ(0x07); //Frame Size = 7 + 1
	  SPI0_CTAR0 = SPI_CTAR_FMSZ(0x07)|SPI_CTAR_PBR(0x01)|SPI_CTAR_BR(0x04)|SPI_CTAR_PCSSCK(0x01)|SPI_CTAR_CSSCK(0x03)|SPI_CTAR_PDT(0x01)|SPI_CTAR_DT(0x01);//|SPI_CTAR_CPOL_MASK|SPI_CTAR_CPHA_MASK;
  /* SPI0_CTAR0 =  SPI_CTAR_FMSZ(0x07) |
                 SPI_CTAR_CPOL_MASK |
                 SPI_CTAR_PCSSCK(0x01) |
                 SPI_CTAR_PASC(0x01) |
                 SPI_CTAR_PDT(0x03) |
                 SPI_CTAR_PBR(0x01) |
                 SPI_CTAR_CSSCK(0x03) |
                 SPI_CTAR_ASC(0x03) |
                 SPI_CTAR_DT(0x01) |
                 SPI_CTAR_BR(0x05);    */  /* Set Clock and Transfer Attributes register */

	SPI0_RSER |= SPI_RSER_RFDF_RE_MASK | SPI_RSER_RFDF_DIRS_MASK |  SPI_RSER_TFFF_DIRS_MASK;   /* Set DMA Interrupt Request Select and Enable register */


	SPI0_MCR &= ~SPI_MCR_HALT_MASK;	//Start transfers.

}

void Master_send(char mstr_val)
{

	SPI0_PUSHR = mstr_val | SPI_PUSHR_EOQ_MASK | SPI_PUSHR_CTCNT_MASK | SPI_PUSHR_PCS(0x01);
	while(!(SPI0_SR & SPI_SR_TFFF_MASK)) {} //wait till TX fifo not full

	while(!(SPI0_SR & SPI_SR_EOQF_MASK)) {} //wait till the las entry has been transmitted
	SPI0_SR  |= SPI_SR_EOQF_MASK; //clear EOQF flag
	SPI0_SR	 |= SPI_SR_TFFF_MASK; //clear TFFF flag

}
char Master_receive ()
{
	char Mreceive_val = 0;
	while (!(SPI0_SR & SPI_SR_RFDF_MASK)) {} //wait till RX fifo is not empty
	Mreceive_val = SPI0_POPR;
	SPI0_SR |= SPI_SR_RFDF_MASK;
	return Mreceive_val;
}









static void init_clock(void)
{
	SIM_SCGC6 |= SIM_SCGC6_SPI0_MASK;
}
static void init_io(void)
{
	  /*PORTA_PCR17 = (uint32_t)((PORTA_PCR17 & (uint32_t)~(uint32_t)(
	                 PORT_PCR_ISF_MASK |
	                 PORT_PCR_MUX(0x05)
	                )) | (uint32_t)(
	                 PORT_PCR_MUX(0x02)
	                ));*/
	PORTD_PCR4 &= ~PORT_PCR_MUX_MASK;
	PORTA_PCR4 |= PORT_PCR_MUX(2);
	PORTA_PCR5 &= ~PORT_PCR_MUX_MASK;
	PORTA_PCR5 |= PORT_PCR_MUX(2);
	PORTA_PCR6 &= ~PORT_PCR_MUX_MASK;
	PORTA_PCR6 |= PORT_PCR_MUX(2);
	PORTA_PCR7 &= ~PORT_PCR_MUX_MASK;
	PORTA_PCR7 |= PORT_PCR_MUX(2);
}
static void init_set_master(void)
{
    SPI0_MCR |= SPI_MCR_MSTR_MASK;
}
static void init_fifo(void)
{
    SPI0_MCR &= ~SPI_MCR_MDIS_MASK;

    SPI0_MCR |= SPI_MCR_DIS_RXF_MASK |
                SPI_MCR_DIS_TXF_MASK |
                SPI_MCR_CLR_RXF_MASK |
                SPI_MCR_CLR_TXF_MASK;

}
static void init_inactive_cs(void)
{
    SPI0_MCR |= SPI_MCR_PCSIS(1<<0);
}
static void init_inactive_clock(void)
{
    // must be low
}
static void init_frame_size(void)
{
    SPI0_CTAR0 &= ~SPI_CTAR_FMSZ_MASK;
    SPI0_CTAR0 |= SPI_CTAR_FMSZ(7);
}
static void init_clock_mode(void)
{
    // defalut = capture data on rising edge
	  SPI0_CTAR0 = SPI_CTAR_CPOL_MASK |
	               SPI_CTAR_CPHA_MASK ;
}
static void init_baudrate(void)
{
    // default = sys clock/2/2 = 48/4 = 12M
	  SPI0_CTAR0 = SPI_CTAR_PBR(0x01) |
	               SPI_CTAR_BR(0x02);      /* Set Clock and Transfer Attributes register */
}
static void init_msb_first(void)
{
    // default is msb first
	 // SPI1_CTAR0 = SPI_CTAR_LSBFE_MASK ;
}
void spi_transfe_start(void)
{
    SPI0_MCR &= ~SPI_MCR_HALT_MASK;
}
void spi_transfe_stop(void)
{
    SPI0_MCR |= SPI_MCR_HALT_MASK;
}

void clear_flags(void)
{
	  SPI0_SR = SPI_SR_TCF_MASK |
	            SPI_SR_TXRXS_MASK |
	            SPI_SR_EOQF_MASK |
	            SPI_SR_TFUF_MASK |
	            SPI_SR_TFFF_MASK |
	            SPI_SR_RFOF_MASK |
	            SPI_SR_RFDF_MASK |
	            SPI_SR_TXCTR(0x00) |
	            SPI_SR_TXNXTPTR(0x00) |
	            SPI_SR_RXCTR(0x00) |
	            SPI_SR_POPNXTPTR(0x00);
}
void spi_delay()
{


	 SPI1_CTAR0 =  SPI_CTAR_PCSSCK(0x00) |
	               SPI_CTAR_PASC(0x00) |
	               SPI_CTAR_PDT(0x00) |
	               SPI_CTAR_CSSCK(0x00)|
	               SPI_CTAR_ASC(0x00) |
	               SPI_CTAR_DT(0x00) ;
}
void spi_init(void)
{
    init_clock();
    init_io();
  /*  init_set_master();
    SPI1_MCR = SPI_MCR_DCONF(0x00) |
               SPI_MCR_PCSIS(0x00) |
               SPI_MCR_SMPL_PT(0x00) |
               SPI_MCR_HALT_MASK;
    init_fifo();
    init_inactive_cs();
    init_frame_size();
    init_clock_mode();
    init_msb_first();
    init_baudrate();
    spi_delay();*/
    SPI0_MCR = SPI_MCR_DCONF(0x00) |
               SPI_MCR_ROOE_MASK |
               SPI_MCR_PCSIS(0x01) |
               SPI_MCR_SMPL_PT(0x00) |
               SPI_MCR_HALT_MASK;        /* Set Configuration register */
    SPI0_MCR = SPI_MCR_MSTR_MASK |
               SPI_MCR_DCONF(0x00) |
               SPI_MCR_ROOE_MASK |
               SPI_MCR_PCSIS(0x01) |
               SPI_MCR_DIS_TXF_MASK |
               SPI_MCR_DIS_RXF_MASK |
               SPI_MCR_CLR_TXF_MASK |
               SPI_MCR_CLR_RXF_MASK |
               SPI_MCR_SMPL_PT(0x00) |
               SPI_MCR_HALT_MASK;        /* Set Configuration register */
    SPI0_CTAR0 = SPI_CTAR_DBR_MASK |
                 SPI_CTAR_FMSZ(0x07) |
                 SPI_CTAR_CPOL_MASK |
                 SPI_CTAR_PCSSCK(0x01) |
                 SPI_CTAR_PASC(0x01) |
                 SPI_CTAR_PDT(0x03) |
                 SPI_CTAR_PBR(0x01) |
                 SPI_CTAR_CSSCK(0x03) |
                 SPI_CTAR_ASC(0x03) |
                 SPI_CTAR_DT(0x01) |
                 SPI_CTAR_BR(0x05);      /* Set Clock and Transfer Attributes register */
    clear_flags();


}
