/*
 * File:        uart.c
 * Purpose:     Provide common UART routines for serial IO
 *
 * Notes:
 *
 */
#include "uart.h"
/********************************************************************/
/*
 * Initialize the UART for 8N1 operation, interrupts disabled, and
 * no hardware flow-control
 *
 * NOTE: Since the UARTs are pinned out in multiple locations on most
 *       Kinetis devices, this driver does not enable UART pin functions.
 *       The desired pins should be enabled before calling this init function.
 *
 * Parameters:
 *  uartch      UART channel to initialize
 *  sysclk      UART module Clock in kHz(used to calculate baud)
 *  baud        UART baud rate
 */
void uart_init (UART_MemMapPtr uartch, int sysclk, int baud)
{
    register unsigned short int sbr, brfa;
    unsigned char temp;

	/* Enable the clock to the selected UART */
    if(uartch == UART0_BASE_PTR)
		SIM_SCGC4 |= SIM_SCGC4_UART0_MASK;
    else
    	if (uartch == UART1_BASE_PTR)
			SIM_SCGC4 |= SIM_SCGC4_UART1_MASK;
    	else
    		if (uartch == UART2_BASE_PTR)
    			SIM_SCGC4 |= SIM_SCGC4_UART2_MASK;
    		else
    			if(uartch == UART3_BASE_PTR)
    				SIM_SCGC4 |= SIM_SCGC4_UART3_MASK;
    			else
    				if(uartch == UART4_BASE_PTR)
    					SIM_SCGC1 |= SIM_SCGC1_UART4_MASK;
    				else
    					SIM_SCGC1 |= SIM_SCGC1_UART5_MASK;

    SIM_SCGC5 |= SIM_SCGC5_PORTE_MASK;	//Port E Clock enabled
    PORTE_PCR9 = PORT_PCR_MUX(0x03); 	//uart5_rx
    PORTE_PCR8 = PORT_PCR_MUX(0x03); 	//uart5_tx

    /* Make sure that the transmitter and receiver are disabled while we
     * change settings.
     */
    UART_C2_REG(uartch) &= ~(UART_C2_TE_MASK| UART_C2_RE_MASK );

    /* Configure the UART for 8-bit mode, parity disabled */
   UART_C1_REG(uartch) = 0;	/* We need all default settings, so entire register is cleared */
 // UART_C1_REG(uartch) = 0x80;//loop mode control enabled

    /* Calculate baud settings */
    sbr = (unsigned short int)((sysclk*1000)/(baud * 16));

    /* Save off the current value of the UARTx_BDH except for the SBR field */
    temp = UART_BDH_REG(uartch) & ~(UART_BDH_SBR(0x1F));

    UART_BDH_REG(uartch) = temp |  UART_BDH_SBR(((sbr & 0x1F00) >> 8));
    UART_BDL_REG(uartch) = (unsigned char)(sbr & UART_BDL_SBR_MASK);

    /* Determine if a fractional divider is needed to get closer to the baud rate */
    brfa = (((sysclk*32000)/(baud * 16)) - (sbr * 32));

    /* Save off the current value of the UARTx_C4 register except for the BRFA field */
    temp = UART_C4_REG(uartch) & ~(UART_C4_BRFA(0x1F));

    UART_C4_REG(uartch) = temp |  UART_C4_BRFA(brfa);

    /* Enable receiver and transmitter */
	UART_C2_REG(uartch) |= (UART_C2_TE_MASK| UART_C2_RE_MASK );

}
/********************************************************************/
/*
 * Wait for a character to be received on the specified UART
 *
 * Parameters:
 *  channel      UART channel to read from
 *
 * Return Values:
 *  the received character
 */
char uart_getchar (UART_MemMapPtr channel)
{

    /* Wait until character has been received */
 // while (!(UART_S1_REG(channel) & UART_S1_RDRF_MASK));
   for(long cnt=0;cnt<0xffffff;cnt++)
   {
   if((UART_S1_REG(channel) & UART_S1_RDRF_MASK))
   break;

   }


   block(0x100);
 //  printf("character received \n");

/* Return the 8-bit data from the receiver */

   return UART_D_REG(channel);
   //UART_C2_REG(UART5_BASE_PTR) |= (UART_C2_RIE_MASK );
}
/********************************************************************/
/*
 * Wait for space in the UART Tx FIFO and then send a character
 *
 * Parameters:
 *  channel      UART channel to send to
 *  ch			 character to send
 */
void uart_putchar (UART_MemMapPtr channel, char ch)
{
    /* Wait until space is available in the FIFO */
 while(!(UART_S1_REG(channel) & UART_S1_TDRE_MASK));

	   	/* Send the character */
    	 UART_D_REG(channel) = (unsigned char)ch;

 }
/********************************************************************/
/*
 * Check to see if a character has been received
 *
 * Parameters:
 *  channel      UART channel to check for a character
 *
 * Return values:
 *  0       No character received
 *  1       Character has been received
 */
void block1( unsigned long value)
{
		 long i;

		    for(i=0;i<value;i++)
    {

    }
}
int uart_getchar_present (UART_MemMapPtr channel)
{
    return (UART_S1_REG(channel) & UART_S1_RDRF_MASK);
}
/********************************************************************/






